@@ -13,6 +13,7 @@
 # under the License.
 
 import datetime
+import types
 
 from six import moves
 import testscenarios
@@ -576,6 +577,22 @@ class TestDatabaseAPI(base.TestCase):
         self.assertNotIn(run_b.id, [x.id for x in result])
         self.assertIn(run_c.id, [x.id for x in result])
 
+    def test_get_recent_runs_by_key_value_metadata_with_start_date(self):
+        run_a = api.create_run(run_at=datetime.datetime(
+            1914, 6, 28, 10, 45, 0))
+        run_b = api.create_run()
+        run_c = api.create_run()
+        api.add_run_metadata({'a_key': 'a_value'}, run_a.id)
+        api.add_run_metadata({'a_key': 'a_value'}, run_c.id)
+        api.add_run_metadata({'a_key': 'b_value'}, run_b.id)
+        result = api.get_recent_runs_by_key_value_metadata(
+            'a_key', 'a_value', start_date=datetime.datetime(
+                1918, 11, 11, 11, 11, 11))
+        self.assertEqual(1, len(result))
+        self.assertNotIn(run_a.id, [x.id for x in result])
+        self.assertNotIn(run_b.id, [x.id for x in result])
+        self.assertIn(run_c.id, [x.id for x in result])
+
     def test_get_recent_runs_by_key_value_metadata_one_run(self):
         timestamp = datetime.datetime(1914, 6, 28, 10, 45, 0)
         run_a = api.create_run(run_at=timestamp)
@@ -641,6 +658,35 @@ class TestDatabaseAPI(base.TestCase):
             'a_key': 'b',
         }, result[0])
 
+    def test_get_test_runs_by_status_for_run_ids_with_run_id(self):
+        run_b = api.create_run(artifacts='fake_url')
+        run_a = api.create_run()
+        run_c = api.create_run()
+        test_a = api.create_test('fake_test')
+        api.add_run_metadata({'a_key': 'b'}, run_b.id)
+        api.add_run_metadata({'a_key': 'a'}, run_a.id)
+        api.add_run_metadata({'a_key': 'c'}, run_c.id)
+        start_timestamp = datetime.datetime(1914, 6, 28, 10, 45, 0)
+        stop_timestamp = datetime.datetime(1914, 6, 28, 10, 50, 0)
+        api.create_test_run(test_a.id, run_a.id, 'success',
+                            datetime.datetime.utcnow())
+        api.create_test_run(test_a.id, run_b.id, 'fail',
+                            start_timestamp, stop_timestamp)
+        api.create_test_run(test_a.id, run_c.id, 'success',
+                            datetime.datetime.utcnow())
+        result = api.get_test_runs_by_status_for_run_ids(
+            'fail', [run_a.uuid, run_b.uuid, run_c.uuid], key='a_key',
+            include_run_id=True)
+        self.assertEqual(1, len(result))
+        self.assertEqual({
+            'test_id': u'fake_test',
+            'link': u'fake_url',
+            'start_time': start_timestamp,
+            'stop_time': stop_timestamp,
+            'a_key': 'b',
+            'uuid': run_b.uuid,
+        }, result[0])
+
     def test_get_all_runs_time_series_by_key_with_overlap(self):
         time_a = datetime.datetime(1914, 6, 28, 10, 45, 0)
         run_a = api.create_run(run_at=time_a)
@@ -815,3 +861,260 @@ class TestDatabaseAPI(base.TestCase):
             'zeon', 'zaku', start_date=datetime.date(1970, 1, 1))
         self.assertEqual(1, len(results))
         self.assertEqual(run_c.id, results[0].id)
+
+    def test_get_test_run_metadata(self):
+        run = api.create_run()
+        test = api.create_test('fake_test')
+        test_run = api.create_test_run(test.id, run.id, 'success',
+                                       datetime.datetime.utcnow(),
+                                       datetime.datetime.utcnow())
+        run_meta = {
+            'key_a': 'value_b',
+            'key_b': 'value_a',
+        }
+        api.add_test_run_metadata(run_meta, test_run.id)
+        test_run_metadata = api.get_test_run_metadata(test_run.id)
+        self.assertEqual(2, len(test_run_metadata))
+        for meta in test_run_metadata:
+            self.assertIn(meta.key, run_meta.keys())
+            self.assertIn(meta.value, run_meta.values())
+
+    def test_get_all_runs_by_date(self):
+        timestamp_a = datetime.datetime.utcnow().replace(microsecond=0)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=20)
+        api.create_run(run_at=timestamp_a)
+        api.create_run(run_at=timestamp_b)
+        api.create_run(run_at=timestamp_c)
+        res = api.get_all_runs_by_date()
+        self.assertEqual(3, len(res))
+        self.assertIn(timestamp_a, [x.run_at for x in res])
+        self.assertIn(timestamp_b, [x.run_at for x in res])
+        self.assertIn(timestamp_c, [x.run_at for x in res])
+        res = api.get_all_runs_by_date(start_date=timestamp_c)
+        self.assertEqual(1, len(res))
+        self.assertEqual(timestamp_c, res[0].run_at)
+        res = api.get_all_runs_by_date(stop_date=timestamp_a)
+        self.assertEqual(1, len(res))
+        self.assertEqual(timestamp_a, res[0].run_at)
+        res = api.get_all_runs_by_date(start_date=timestamp_a,
+                                       stop_date=timestamp_b)
+        self.assertEqual(2, len(res))
+        self.assertIn(timestamp_a, [x.run_at for x in res])
+        self.assertIn(timestamp_b, [x.run_at for x in res])
+
+    def test_get_latest_run(self):
+        timestamp_a = datetime.datetime.utcnow().replace(microsecond=0)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=20)
+        api.create_run(run_at=timestamp_a)
+        api.create_run(run_at=timestamp_b)
+        api.create_run(run_at=timestamp_c)
+        res = api.get_latest_run()
+        self.assertEqual(timestamp_c, res.run_at)
+
+    def test_get_failing_from_run(self):
+        timestamp_a = datetime.datetime.utcnow().replace(microsecond=0)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=20)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        test_a = api.create_test('fake_test')
+        test_b = api.create_test('fake_test2')
+        api.create_test_run(test_a.id, run_a.id, 'fail',
+                            start_time=timestamp_a)
+        api.create_test_run(test_a.id, run_b.id, 'fail',
+                            start_time=timestamp_a)
+        api.create_test_run(test_b.id, run_b.id, 'fail',
+                            start_time=timestamp_b)
+        api.create_test_run(test_b.id, run_a.id, 'success',
+                            start_time=timestamp_c)
+        res = api.get_failing_from_run(run_a.id)
+        self.assertEqual(1, len(res))
+        self.assertEqual(test_a.id, res[0].test_id)
+        self.assertEqual(timestamp_a, res[0].start_time)
+        res = api.get_failing_from_run(run_b.id)
+        self.assertEqual(2, len(res))
+        self.assertEqual('fail', res[0].status)
+        self.assertIn(timestamp_a, [x.start_time for x in res])
+        self.assertIn(timestamp_b, [x.start_time for x in res])
+
+    def test_get_test_run_time_series(self):
+        timestamp_a = datetime.datetime.utcnow().replace(microsecond=0)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=15)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        test = api.create_test('fake_test')
+        api.create_test_run(test.id, run_a.id, 'success',
+                            start_time=timestamp_a,
+                            end_time=timestamp_b)
+        api.create_test_run(test.id, run_b.id, 'success',
+                            start_time=timestamp_b,
+                            end_time=timestamp_c)
+        res = api.get_test_run_time_series(test.id)
+        self.assertEqual(res[timestamp_a], 600)
+        self.assertEqual(res[timestamp_b], 300)
+
+    def test_get_test_status_time_series(self):
+        timestamp_a = datetime.datetime.utcnow().replace(microsecond=0)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=20)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        test = api.create_test('fake_test')
+        api.create_test_run(test.id, run_a.id, 'success',
+                            start_time=timestamp_a,
+                            end_time=timestamp_b)
+        api.create_test_run(test.id, run_b.id, 'fail',
+                            start_time=timestamp_b,
+                            end_time=timestamp_c)
+        res = api.get_test_status_time_series(test.id)
+        self.assertEqual(res[timestamp_a], 'success')
+        self.assertEqual(res[timestamp_b], 'fail')
+
+    def test_get_recent_failed_runs(self):
+        run_a = api.create_run(fails=1)
+        api.create_run()
+        run_c = api.create_run(fails=2)
+        run_d = api.create_run(fails=1)
+        res = api.get_recent_failed_runs()
+        self.assertEqual(3, len(res))
+        self.assertIn(run_a.uuid, res)
+        self.assertIn(run_c.uuid, res)
+        self.assertIn(run_d.uuid, res)
+
+    def test_get_recent_failed_runs_with_start_date(self):
+        api.create_run(fails=1, run_at=datetime.datetime(
+            1914, 6, 28, 10, 45, 0))
+        api.create_run()
+        run_c = api.create_run(fails=2)
+        run_d = api.create_run(fails=1)
+        res = api.get_recent_failed_runs(start_date=datetime.datetime(
+            1918, 11, 11, 11, 11, 11))
+        self.assertEqual(2, len(res))
+        self.assertIn(run_c.uuid, res)
+        self.assertIn(run_d.uuid, res)
+
+    def test_get_recent_successful_runs(self):
+        run_a = api.create_run(passes=1)
+        run_b = api.create_run()
+        run_c = api.create_run(passes=2)
+        api.create_run(fails=1)
+        res = api.get_recent_successful_runs()
+        self.assertEqual(3, len(res))
+        self.assertIn(run_a.uuid, res)
+        self.assertIn(run_b.uuid, res)
+        self.assertIn(run_c.uuid, res)
+
+    def test_get_recent_successful_runs_with_start_date(self):
+        api.create_run(passes=1, run_at=datetime.datetime(
+            1914, 6, 28, 10, 45, 0))
+        run_b = api.create_run()
+        run_c = api.create_run(passes=2)
+        api.create_run(fails=1)
+        res = api.get_recent_successful_runs(start_date=datetime.datetime(
+            1918, 11, 11, 11, 11, 11))
+        self.assertEqual(2, len(res))
+        self.assertIn(run_b.uuid, res)
+        self.assertIn(run_c.uuid, res)
+
+    def test_get_test_counts_in_date_range_as_str(self):
+        timestamp_str_a = 'Dec 01 2015'
+        timestamp_str_b = 'Dec 20 2015'
+        timestamp_a = datetime.datetime(2015, 12, 2, 10, 00, 00)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=20)
+        timestamp_d = datetime.datetime(2015, 12, 22, 10, 00, 00)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        run_c = api.create_run()
+        run_d = api.create_run()
+        test = api.create_test('fake_test')
+        api.create_test_run(test.id, run_a.id, 'success',
+                            timestamp_a, timestamp_b)
+        api.create_test_run(test.id, run_b.id, 'fail',
+                            timestamp_a, timestamp_c)
+        api.create_test_run(test.id, run_c.id, 'success',
+                            timestamp_a, timestamp_d)
+        api.create_test_run(test.id, run_d.id, 'skip',
+                            timestamp_c, timestamp_d)
+        res = api.get_test_counts_in_date_range(test.id,
+                                                timestamp_str_a,
+                                                timestamp_str_b)
+        self.assertEqual(1, res['success'])
+        self.assertEqual(1, res['failure'])
+        self.assertEqual(0, res['skips'])
+
+    def test_get_test_counts_in_date_range(self):
+        timestamp_a = datetime.datetime(2015, 12, 2, 10, 00, 00)
+        timestamp_b = timestamp_a + datetime.timedelta(minutes=10)
+        timestamp_c = timestamp_a + datetime.timedelta(minutes=20)
+        timestamp_d = datetime.datetime(2015, 12, 22, 10, 00, 00)
+        timerange_a = datetime.datetime(2015, 12, 1)
+        timerange_b = datetime.datetime(2015, 12, 20)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        run_c = api.create_run()
+        run_d = api.create_run()
+        test = api.create_test('fake_test')
+        api.create_test_run(test.id, run_a.id, 'success',
+                            timestamp_a, timestamp_b)
+        api.create_test_run(test.id, run_b.id, 'fail',
+                            timestamp_a, timestamp_c)
+        api.create_test_run(test.id, run_c.id, 'success',
+                            timestamp_a, timestamp_d)
+        api.create_test_run(test.id, run_d.id, 'skip',
+                            timestamp_c, timestamp_d)
+        res = api.get_test_counts_in_date_range(test.id,
+                                                timerange_a,
+                                                timerange_b)
+        self.assertEqual(1, res['success'])
+        self.assertEqual(1, res['failure'])
+        self.assertEqual(0, res['skips'])
+
+    def test_get_failing_test_ids_from_runs_by_key_value(self):
+        test_a = api.create_test('fake_test')
+        test_b = api.create_test('fake_test1')
+        test_c = api.create_test('fake_test2')
+        test_d = api.create_test('fake_test3')
+        run_a = api.create_run()
+        run_b = api.create_run()
+        run_c = api.create_run()
+        api.add_run_metadata({'a_key': 'a_value'}, run_a.id)
+        api.add_run_metadata({'a_key': 'a_value'}, run_b.id)
+        api.add_run_metadata({'a_key': 'b_value'}, run_c.id)
+        api.create_test_run(test_a.id, run_a.id, 'fail')
+        api.create_test_run(test_b.id, run_b.id, 'success')
+        api.create_test_run(test_c.id, run_b.id, 'fail')
+        api.create_test_run(test_d.id, run_c.id, 'fail')
+        res = api.get_failing_test_ids_from_runs_by_key_value('a_key',
+                                                              'a_value')
+        self.assertEqual(2, len(res))
+        self.assertIn(test_a.test_id, res)
+        self.assertIn(test_c.test_id, res)
+
+    def test_add_test_run_attachments(self):
+        test = api.create_test('fake_test')
+        run = api.create_run()
+        test_run = api.create_test_run(test.id, run.id, 'success')
+        attach_dict = {'attach_label': b'attach',
+                       'attach_label_a': b'attach_a'}
+        res = api.add_test_run_attachments(attach_dict, test_run.id)
+        self.assertEqual(2, len(res))
+        self.assertEqual(test_run.id, res[0].test_run_id)
+        self.assertIn('attach_label', [x.label for x in res])
+        self.assertIn('attach_label_a', [x.label for x in res])
+        self.assertIn(b'attach', [x.attachment for x in res])
+        self.assertIn(b'attach_a', [x.attachment for x in res])
+
+    def test_get_ids_for_all_tests(self):
+        test_a = api.create_test('fake_test')
+        test_b = api.create_test('fake_test1')
+        test_c = api.create_test('fake_test2')
+        res = api.get_ids_for_all_tests()
+        self.assertIsInstance(res, types.GeneratorType)
+        res = list(res)
+        self.assertIn((test_c.id,), res)
+        self.assertIn((test_b.id,), res)
+        self.assertIn((test_a.id,), res)
